From: Yinghai Lu <yinghai@kernel.org>
To: "H. Peter Anvin" <hpa@zytor.com>, Ingo Molnar <mingo@elte.hu>,
Andrew Morton <akpm@linux-foundation.org>,
Thomas Gleixner <tglx@linutronix.de>,
Jeremy Fitzhardinge <jeremy.fitzhardinge@citrix.com>
Cc: Linux Kernel Mailing List <linux-kernel@vger.kernel.org>
Subject: [PATCH] x86: put initial_pg_tables into .bss -v4
Date: Mon, 09 Mar 2009 01:15:57 -0700 [thread overview]
Message-ID: <49B4D03D.7030205@kernel.org> (raw)
In-Reply-To: <49AB2E69.1010201@kernel.org>
Impact: cleanup
Don't use ram after _end blindly for pagetables. aka init pages is before _end
put those pg table into .bss
v2: keep initial page table up to 512M only.
v4: put initial page tables just before _end
Signed-off-by: Yinghai Lu <yinghai@kernel.org>
---
arch/x86/include/asm/page_32_types.h | 5 +++
arch/x86/kernel/head32.c | 3 +
arch/x86/kernel/head_32.S | 55 ++++++++++++++---------------------
arch/x86/kernel/vmlinux_32.lds.S | 11 ++++++-
4 files changed, 40 insertions(+), 34 deletions(-)
Index: linux-2.6/arch/x86/kernel/head32.c
===================================================================
--- linux-2.6.orig/arch/x86/kernel/head32.c
+++ linux-2.6/arch/x86/kernel/head32.c
@@ -18,7 +18,8 @@ void __init i386_start_kernel(void)
{
reserve_trampoline_memory();
- reserve_early(__pa_symbol(&_text), __pa_symbol(&_end), "TEXT DATA BSS");
+ reserve_early(__pa_symbol(&_text), __pa_symbol(&__bss_stop),
+ "TEXT DATA BSS");
#ifdef CONFIG_BLK_DEV_INITRD
/* Reserve INITRD */
Index: linux-2.6/arch/x86/kernel/vmlinux_32.lds.S
===================================================================
--- linux-2.6.orig/arch/x86/kernel/vmlinux_32.lds.S
+++ linux-2.6/arch/x86/kernel/vmlinux_32.lds.S
@@ -189,10 +189,13 @@ SECTIONS
*(.bss)
. = ALIGN(4);
__bss_stop = .;
- _end = . ;
+ /* extra_page_aligned must be last one before end*/
/* This is where the kernel creates the early boot page tables */
. = ALIGN(PAGE_SIZE);
pg0 = . ;
+ *(.bss.extra_page_aligned)
+ . = ALIGN(8);
+ _end = . ;
}
/* Sections to be discarded */
@@ -205,6 +208,12 @@ SECTIONS
DWARF_DEBUG
}
+/*
+ * Build-time check on the image size:
+ */
+ASSERT((_end - LOAD_OFFSET <= KERNEL_IMAGE_SIZE),
+ "kernel image bigger than KERNEL_IMAGE_SIZE")
+
#ifdef CONFIG_KEXEC
/* Link time checks */
#include <asm/kexec.h>
Index: linux-2.6/arch/x86/include/asm/page_32_types.h
===================================================================
--- linux-2.6.orig/arch/x86/include/asm/page_32_types.h
+++ linux-2.6/arch/x86/include/asm/page_32_types.h
@@ -39,6 +39,11 @@
#define __VIRTUAL_MASK_SHIFT 32
#endif /* CONFIG_X86_PAE */
+/*
+ * Kernel image size is limited to 512 MB (see in arch/x86/kernel/head_32.S)
+ */
+#define KERNEL_IMAGE_SIZE (512 * 1024 * 1024)
+
#ifndef __ASSEMBLY__
/*
next prev parent reply other threads:[~2009-03-09 8:17 UTC|newest]
Thread overview: 43+ messages / expand[flat|nested] mbox.gz Atom feed top
2009-02-28 1:51 [PATCH] Simple brk allocator for very early allocations Jeremy Fitzhardinge
2009-02-28 1:51 ` [PATCH] x86: add brk allocation for very, " Jeremy Fitzhardinge
2009-02-28 1:51 ` [PATCH] x86: reserve brk earlier Jeremy Fitzhardinge
2009-02-28 1:51 ` [PATCH] x86-32: use brk segment for allocating initial kernel pagetable Jeremy Fitzhardinge
2009-02-28 7:02 ` Yinghai Lu
2009-02-28 7:05 ` J Jeremy Fitzhardinge
2009-02-28 7:15 ` J Ingo Molnar
2009-02-28 7:39 ` does boot loader check uncompressed kernel size? Yinghai Lu
2009-02-28 7:47 ` Cyrill Gorcunov
2009-02-28 7:54 ` Yinghai Lu
2009-02-28 8:08 ` H. Peter Anvin
2009-02-28 20:42 ` Yinghai Lu
2009-02-28 7:52 ` brk patches Yinghai Lu
2009-02-28 8:08 ` H. Peter Anvin
2009-02-28 8:17 ` Jeremy Fitzhardinge
2009-02-28 20:40 ` Yinghai Lu
2009-03-01 23:53 ` Jeremy Fitzhardinge
2009-03-02 1:02 ` Yinghai Lu
2009-03-02 1:07 ` H. Peter Anvin
2009-03-02 1:16 ` Jeremy Fitzhardinge
2009-03-02 1:36 ` H. Peter Anvin
2009-03-02 1:54 ` Jeremy Fitzhardinge
2009-03-02 2:12 ` Yinghai Lu
2009-03-01 1:23 ` [PATCH] x86: put initial_pg_tables into bss Yinghai Lu
2009-03-01 8:31 ` [PATCH] x86: put initial_pg_tables into bss -v2 Yinghai Lu
2009-03-01 9:20 ` H. Peter Anvin
2009-03-01 17:49 ` Yinghai Lu
2009-03-01 18:06 ` Yinghai Lu
2009-03-01 23:29 ` H. Peter Anvin
2009-03-02 0:55 ` Yinghai Lu
2009-03-09 8:15 ` Yinghai Lu [this message]
2009-03-09 15:41 ` [PATCH] x86: put initial_pg_tables into .bss -v4 H. Peter Anvin
2009-03-09 17:35 ` Yinghai Lu
2009-03-09 18:28 ` H. Peter Anvin
2009-03-11 1:39 ` Jeremy Fitzhardinge
2009-03-09 7:45 ` [PATCH] x86: put initial_pg_tables into bss -v2 Yinghai Lu
2009-02-28 8:07 ` does boot loader check uncompressed kernel size? H. Peter Anvin
2009-02-28 8:17 ` J Jeremy Fitzhardinge
2009-02-28 7:30 ` J Yinghai Lu
2009-02-28 1:51 ` [PATCH] x86: use brk allocation for DMI Jeremy Fitzhardinge
2009-02-28 1:51 ` [PATCH] x86: leave _brk_end defined Jeremy Fitzhardinge
2009-02-28 5:23 ` [PATCH] Simple brk allocator for very early allocations Andrew Morton
2009-02-28 6:30 ` Jeremy Fitzhardinge
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=49B4D03D.7030205@kernel.org \
--to=yinghai@kernel.org \
--cc=akpm@linux-foundation.org \
--cc=hpa@zytor.com \
--cc=jeremy.fitzhardinge@citrix.com \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@elte.hu \
--cc=tglx@linutronix.de \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.
Index: linux-2.6/arch/x86/kernel/head_32.S
===================================================================
--- linux-2.6.orig/arch/x86/kernel/head_32.S
+++ linux-2.6/arch/x86/kernel/head_32.S
@@ -38,42 +38,30 @@
#define X86_VENDOR_ID new_cpu_data+CPUINFO_x86_vendor_id
/*
- * This is how much memory *in addition to the memory covered up to
- * and including _end* we need mapped initially.
- * We need:
- * - one bit for each possible page, but only in low memory, which means
- * 2^32/4096/8 = 128K worst case (4G/4G split.)
+ * This is how much memory for page table to and including _end
+ * we need mapped initially.
* - enough space to map all low memory, which means
* (2^32/4096) / 1024 pages (worst case, non PAE)
* (2^32/4096) / 512 + 4 pages (worst case for PAE)
* - a few pages for allocator use before the kernel pagetable has
* been set up
*
* Modulo rounding, each megabyte assigned here requires a kilobyte of
* memory, which is currently unreclaimed.
*
* This should be a multiple of a page.
+ *
+ * KERNEL_IMAGE_SIZE should be greater than pa(_end)
+ * and small than max_low_pfn, otherwise will waste some page table entries
*/
-LOW_PAGES = 1<<(32-PAGE_SHIFT_asm)
-
-/*
- * To preserve the DMA pool in PAGEALLOC kernels, we'll allocate
- * pagetables from above the 16MB DMA limit, so we'll have to set
- * up pagetables 16MB more (worst-case):
- */
-#if defined(CONFIG_DEBUG_PAGEALLOC) || defined(CONFIG_KMEMCHECK)
-LOW_PAGES = LOW_PAGES + 0x1000000
-#endif
+LOW_PAGES = (KERNEL_IMAGE_SIZE + PAGE_SIZE_asm - 1)>>PAGE_SHIFT
#if PTRS_PER_PMD > 1
PAGE_TABLE_SIZE = (LOW_PAGES / PTRS_PER_PMD) + PTRS_PER_PGD
#else
PAGE_TABLE_SIZE = (LOW_PAGES / PTRS_PER_PGD)
#endif
-BOOTBITMAP_SIZE = LOW_PAGES / 8
ALLOCATOR_SLOP = 4
-INIT_MAP_BEYOND_END = BOOTBITMAP_SIZE + (PAGE_TABLE_SIZE + ALLOCATOR_SLOP)*PAGE_SIZE_asm
+INIT_MAP_SIZE = (PAGE_TABLE_SIZE + ALLOCATOR_SLOP) * PAGE_SIZE_asm
/*
* 32-bit kernel entrypoint; only used by the boot CPU. On entry,
@@ -166,10 +154,9 @@ num_subarch_entries = (. - subarch_entri
/*
* Initialize page tables. This creates a PDE and a set of page
- * tables, which are located immediately beyond _end. The variable
- * init_pg_tables_end is set up to point to the first "safe" location.
+ * tables, which are located immediately beyond _end.
* Mappings are created both at virtual address 0 (identity mapping)
- * and PAGE_OFFSET for up to _end+sizeof(page tables)+INIT_MAP_BEYOND_END.
+ * and PAGE_OFFSET for up to _end
*
* Note that the stack is not yet set up!
*/
@@ -209,14 +196,14 @@ default_entry:
loop 11b
/*
- * End condition: we must map up to and including INIT_MAP_BEYOND_END
- * bytes beyond the end of our own page tables.
+ * End condition: we must map up to the end.
*/
- leal (INIT_MAP_BEYOND_END+PTE_IDENT_ATTR)(%edi),%ebp
+ movl $pa(_end), %ebp
+ addl PTE_IDENT_ATTR, %ebp
cmpl %ebp,%eax
jb 10b
1:
- movl %edi,pa(init_pg_tables_end)
+ movl %edi, pa(init_pg_tables_end)
shrl $12, %eax
movl %eax, pa(max_pfn_mapped)
@@ -242,14 +229,14 @@ page_pde_offset = (__PAGE_OFFSET >> 20);
addl $0x1000,%eax
loop 11b
/*
- * End condition: we must map up to and including INIT_MAP_BEYOND_END
- * bytes beyond the end of our own page tables; the +0x007 is
+ * End condition: we must map up to end, the +0x007 is
* the attribute bits
*/
- leal (INIT_MAP_BEYOND_END+PTE_IDENT_ATTR)(%edi),%ebp
+ movl $pa(_end), %ebp
+ addl PTE_IDENT_ATTR, %ebp
cmpl %ebp,%eax
jb 10b
- movl %edi,pa(init_pg_tables_end)
+ movl %edi, pa(init_pg_tables_end)
shrl $12, %eax
movl %eax, pa(max_pfn_mapped)
@@ -636,6 +623,10 @@ swapper_pg_fixmap:
.fill 1024,4,0
ENTRY(empty_zero_page)
.fill 4096,1,0
+
+.section ".bss.extra_page_aligned","wa"
+ .align PAGE_SIZE_asm
+ .fill INIT_MAP_SIZE,1,0
/*
* This starts the data section.
*/
@@ -38,42 +38,30 @@
* - enough space to map all low memory, which means
- * (2^32/4096) / 1024 pages (worst case, non PAE)
- * (2^32/4096) / 512 + 4 pages (worst case for PAE)
- * - a few pages for allocator use before the kernel pagetable has
- * been set up
+ * (KERNEL_IMAGE_SIZE/4096) / 1024 pages (worst case, non PAE)
+ * (KERNEL_IMAGE_SIZE/4096) / 512 + 4 pages (worst case for PAE)
*
* Modulo rounding, each megabyte assigned here requires a kilobyte of